@@ -273,6 +273,10 @@ do_recv(LogFun, RecvPid, SeqNum) when is_function(LogFun);
 	{p1_mysql_recv, RecvPid, closed, _E} ->
 	    p1_mysql:log(LogFun, error, "p1_mysql_conn: p1_mysql_recv:"
 		      " socket was closed ~p~n", [{RecvPid, _E}]),
+	    {error, "p1_mysql_recv: socket was closed"};
+	close ->
+	    p1_mysql:log(LogFun, error, "p1_mysql_conn: p1_mysql_recv:"
+					" received close~n", []),
 	    {error, "p1_mysql_recv: socket was closed"}
     end;
 do_recv(LogFun, RecvPid, SeqNum) when is_function(LogFun);
@@ -287,6 +291,10 @@ do_recv(LogFun, RecvPid, SeqNum) when is_function(LogFun);
 	{p1_mysql_recv, RecvPid, closed, _E} ->
 	    p1_mysql:log(LogFun, error, "p1_mysql_conn: p1_mysql_recv:"
 		      " socket was closed 2 ~p~n", [{RecvPid, _E}]),
+	    {error, "p1_mysql_recv: socket was closed"};
+	close ->
+	    p1_mysql:log(LogFun, error, "p1_mysql_conn: p1_mysql_recv:"
+					" received close~n", []),
 	    {error, "p1_mysql_recv: socket was closed"}
         end.
 
@@ -372,7 +380,14 @@ loop(State) ->
 		    erlang:cancel_timer(Ref),
 		    gen_server:reply(GenSrvFrom, Res)
 	    end,
-	    loop(State);
+	    case Res of
+		{error, #p1_mysql_result{error="p1_mysql_recv: socket was closed"}} ->
+		    p1_mysql:log(State#state.log_fun, error, "p1_mysql_conn: "
+							     "Connection closed, exiting.", []),
+		    close_connection(State);
+		_ ->
+		    loop(State)
+	    end;
 	{p1_mysql_recv, RecvPid, data, Packet, Num} ->
 	    p1_mysql:log(State#state.log_fun, error, "p1_mysql_conn: "
 		      "Received MySQL data when not expecting any "
@@ -386,7 +401,7 @@ loop(State) ->
                          "Connection closed, exiting.", []),
             close_connection(State);
 	close ->
-	    p1_mysql:log(State#state.log_fun, error, "p1_mysql_conn: "
+	    p1_mysql:log(State#state.log_fun, info, "p1_mysql_conn: "
 		      "Received close signal, exiting.", []),
 	    close_connection(State);
         Unknown ->
